﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators.LCS {
  [Item("MDLIterationOperator", "Description missing")]
  [StorableClass]
  public class MDLIterationOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public IValueLookupParameter<MDLCalculator> MDLCalculatorParameter {
      get { return (IValueLookupParameter<MDLCalculator>)Parameters["MDLCalculator"]; }
    }
    public ILookupParameter<IntValue> IterationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Iterations"]; }
    }
    public ILookupParameter<IntValue> MDLActivationIterationParameter {
      get { return (ILookupParameter<IntValue>)Parameters["MDLActivationIteration"]; }
    }
    public ILookupParameter<DoubleValue> InitialTheoryLengthRatioParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["InitialTheoryLengthRatio"]; }
    }
    public ILookupParameter<DoubleValue> WeightRelaxFactorParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["WeightRelaxFactor"]; }
    }
    public ILookupParameter<IntValue> WeightAdaptionIterationsParameter {
      get { return (ILookupParameter<IntValue>)Parameters["WeightAdaptionIterations"]; }
    }

    public ScopeTreeLookupParameter<IGAssistIndividual> IndividualParameter {
      get { return (ScopeTreeLookupParameter<IGAssistIndividual>)Parameters["Individual"]; }
    }
    public LookupParameter<IGAssistProblemData> ProblemDataParameter {
      get { return (LookupParameter<IGAssistProblemData>)Parameters["ProblemData"]; }
    }
    public IScopeTreeLookupParameter<DoubleValue> QualityParameter {
      get { return (IScopeTreeLookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<BoolValue> MaximizationParameter {
      get { return (ILookupParameter<BoolValue>)Parameters["Maximization"]; }
    }
    #endregion

    [StorableConstructor]
    protected MDLIterationOperator(bool deserializing) : base(deserializing) { }
    protected MDLIterationOperator(MDLIterationOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public MDLIterationOperator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("Iterations", ""));
      Parameters.Add(new ValueLookupParameter<MDLCalculator>("MDLCalculator", ""));
      Parameters.Add(new LookupParameter<IntValue>("MDLActivationIteration", ""));
      Parameters.Add(new LookupParameter<DoubleValue>("InitialTheoryLengthRatio", ""));
      Parameters.Add(new LookupParameter<DoubleValue>("WeightRelaxFactor", ""));
      Parameters.Add(new LookupParameter<IntValue>("WeightAdaptionIterations", ""));

      Parameters.Add(new ScopeTreeLookupParameter<IGAssistIndividual>("Individual", ""));
      Parameters.Add(new LookupParameter<IGAssistProblemData>("ProblemData", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Quality", "The qualities of the trees that should be analyzed."));
      Parameters.Add(new LookupParameter<BoolValue>("Maximization", "The direction of optimization."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MDLIterationOperator(this, cloner);
    }

    public override IOperation Apply() {
      if (MDLCalculatorParameter.ActualValue == null) {
        MDLCalculatorParameter.ActualValue = new MDLCalculator(MDLActivationIterationParameter.ActualValue.Value,
                                                               InitialTheoryLengthRatioParameter.ActualValue.Value,
                                                               WeightRelaxFactorParameter.ActualValue.Value,
                                                               WeightAdaptionIterationsParameter.ActualValue.Value);
      } else {
        var qualities = QualityParameter.ActualValue;
        var bestQuality = MaximizationParameter.ActualValue.Value ? qualities.Max() : qualities.Min();
        int index = qualities.IndexOf(bestQuality);
        var dls = IndividualParameter.ActualValue[index].CreateGAssistSolution(ProblemDataParameter.ActualValue);

        MDLCalculatorParameter.ActualValue.StartNewIteration(dls, IterationsParameter.ActualValue.Value);
      }
      return base.Apply();
    }
  }
}
